@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Hector Peraza.
 
     This program is free software; you can redistribute it and/or modify
@@ -24,10 +24,10 @@
 
 
 #define XCLASS_MAJOR_VERSION  "0"
-#define XCLASS_MINOR_VERSION  "6"
-#define XCLASS_PATCH_LEVEL    "2"
-#define XCLASS_VERSION        "0.6.2"
-#define XCLASS_RELEASE_DATE   "27-Nov-2001"
+#define XCLASS_MINOR_VERSION  "8"
+#define XCLASS_PATCH_LEVEL    "3"
+#define XCLASS_VERSION        "0.8.3"
+#define XCLASS_RELEASE_DATE   "5-Jul-2004"
 
 
-#endif // __VERSION_H
+#endif  // __VERSION_H
