@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Ricky Ralston, Hector Peraza.
 
     This library is free software; you can redistribute it and/or
@@ -650,8 +650,18 @@ int OResourcePool::Reload() {
 
   _focusGC->SetBackground(_selBgnd);
 
-  //_checkered = ...
+  // update _checkered pixmap
 
+  XGCValues gval;
+  unsigned long gmask;
+
+  XFillRectangle(_dpy, _checkered, _whiteGC->GetGC(), /* _hiliteGC->GetGC() */
+                 0, 0, gray_width, gray_height);
+  _focusGC->SetForeground(_frameBgnd);
+  XFillRectangle(_dpy, _checkered, _focusGC->GetGC(),
+                 0, 0, gray_width, gray_height);
+  _focusGC->SetForeground(_black);
+  
   return _changeMask;
 }
 
@@ -682,25 +692,24 @@ char *OResourcePool::FindIniFile(const char *name, int mode) const {
     sprintf(inipath, "%s/etc/%s", _userRoot, name);
     if (access(inipath, R_OK) == 0) return inipath;
 
-    // 2: if not found, search in ${_systemRoot}/etc
-
-    sprintf(inipath, "%s/etc/%s", _systemRoot, name);
-    if (access(inipath, R_OK) == 0) return inipath;
-
-    // 3: if still not found, try just ${HOME}/.name
+    // 2: if not found, try just ${HOME}/.name
 
     sprintf(inipath, "%s/%s%s", _homeDir, (*name == '.') ? "" : ".", name);
     if (access(inipath, R_OK) == 0) return inipath;
 
+    // 3: if still not found, search in ${_systemRoot}/etc
+
+    sprintf(inipath, "%s/etc/%s", _systemRoot, name);
+    if (access(inipath, R_OK) == 0) return inipath;
+
     // 4: not found, return NULL...
+
     delete[] inipath;
     return NULL;
 
   } else {
 
-    // if writing, we'll only attemp to do it in ${_userRoot}/etc
-    // note that we do not check for the existence of the file, since
-    // this could be the first time the file is being created
+    // we'll only attempt to save the file in ${_userRoot}/etc
 
     sprintf(inipath, "%s/etc", _userRoot);
 
@@ -709,8 +718,7 @@ char *OResourcePool::FindIniFile(const char *name, int mode) const {
       return inipath;
     }
 
-    // the _userRoot directory apparently doesn't exist yet, lets
-    // create it
+    // the _userRoot directory apparently doesn't exist yet, create it
 
     if (MakePath(inipath, 0777) == 0) {
       sprintf(inipath, "%s/etc/%s", _userRoot, name);
@@ -724,3 +732,44 @@ char *OResourcePool::FindIniFile(const char *name, int mode) const {
 
   }
 }
+
+char *OResourcePool::FindHelpFile(const char *name, const char *appname) const {
+  char *help_path = new char[PATH_MAX];
+
+  // 1: search for the help file in ${_userRoot}/docs/appname/
+
+  sprintf(help_path, "%s/docs/%s/%s", _userRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+  // 2: if not found try in ${_userRoot}/apps/docs/appname/
+
+  sprintf(help_path, "%s/apps/docs/%s/%s", _userRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+  // 3: if not found, try just in ${_userRoot}/apps/appname/
+
+  sprintf(help_path, "%s/apps/%s/%s", _userRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+
+  // 4: if nothing found so far, try then ${_systemRoot}/docs/appname
+
+  sprintf(help_path, "%s/docs/%s/%s", _systemRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+  // 5: if not found try in ${_systemRoot}/apps/docs/appname/
+
+  sprintf(help_path, "%s/apps/docs/%s/%s", _systemRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+  // 6: if still not found, try in ${_systemRoot}/apps/appname/
+
+  sprintf(help_path, "%s/apps/%s/%s", _systemRoot, appname, name);
+  if (access(help_path, R_OK) == 0) return help_path;
+
+
+  // 7: nothing found, return NULL...
+
+  delete[] help_path;
+  return NULL;
+}
