@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Ricky Ralston, Hector Peraza.
 
     This library is free software; you can redistribute it and/or
@@ -66,6 +66,8 @@ OXClient::OXClient(int argc, char *argv[]) {
 void OXClient::_Init(const char *DpyName) {
   XWindowAttributes root_attr;
 
+  _Root = NULL;
+
   //--- Open the connection to the display
 
   if (!(_dpy = XOpenDisplay(DpyName)))
@@ -128,14 +130,16 @@ OXClient::~OXClient() {
 
   Debug(DBG_MISC, "OXClient: terminating!\n");
 
+  delete _resourcePool;
+
+  delete _Root;        // before _wlist gets destroyed!
+
   delete _wlist;
   delete _tlist;
   delete _idlelist;
   delete _iolist;
   delete _siglist;
 
-  delete _resourcePool;
-
   XSync(GetDisplay(), False);
 
   XCloseDisplay(_dpy); // this should do a cleanup of the remaining
@@ -343,6 +347,11 @@ void OXClient::FreeFont(OXFont *font) {
 // Register / unregister windows...
 
 int OXClient::RegisterWindow(OXWindow *w, char *name) {
+  if (_Root && (w->_id == _Root->_id)) {
+    // someone has created a new OXRootWindow object...
+    delete _Root;
+    _Root = w;
+  }
   return (_wlist->Add(w->_id, (XPointer) w) != NULL);
 }
 
@@ -421,7 +430,11 @@ void OXClient::WaitFor(OXWindow *w) {
     _tip_timer = NULL;
   }
   wid = w->GetId();
-  while (ProcessOneEvent(DestroyNotify, wid));
+  while (ProcessOneEvent(DestroyNotify, wid)) {
+    // safety check!
+    const OXWindow *w1 = GetWindowById(wid);
+    if (!w1 || (w1 != w)) break;
+  }
   _tip_status = 0;
 }
 
@@ -434,7 +447,11 @@ void OXClient::WaitForUnmap(OXWindow *w) {
   //  _tip_timer = NULL;
   //  }
   wid = w->GetId();
-  while (ProcessOneEvent(UnmapNotify, wid));
+  while (ProcessOneEvent(UnmapNotify, wid)) {
+    // safety check!
+    const OXWindow *w1 = GetWindowById(wid);
+    if (!w1 || (w1 != w)) break;
+  }
   //_tip_status = 0;
 }
 
@@ -679,6 +696,15 @@ int OXClient::_DoRedraw() {
   return True;
 }
 
+void OXClient::FlushRedraw() {
+#if 1
+  XEvent event;
+  while (XCheckMaskEvent(GetDisplay(), ExposureMask, &event))
+    HandleEvent(&event);
+#endif
+  _DoRedraw();
+  XSync(GetDisplay(), False);
+}
 
 int OXClient::HandleEvent(XEvent *Event) {
   OXWindow *w;
@@ -712,12 +738,16 @@ int OXClient::HandleMaskEvent(XEvent *Event, Window wid) {
     // This breaks class member protection, but OXClient is a friend of all
     // classes and _should_ know what to do and what *not* to do...
 
-    for (ptr = w; ptr->_parent != NULL; ptr = (OXWindow *) ptr->_parent) {
+    for (ptr = w; ptr != NULL; ptr = (OXWindow *) ptr->_parent) {
       if (ptr->_id == wid) {
         w->HandleEvent(Event);
         return True;
       }
     }
+    if (w->HandleMaskEvent(Event)) {
+      w->HandleEvent(Event);
+      return True;
+    }
   }
 
   if (Event->type == ButtonPress ||
