@@ -324,7 +324,7 @@ void OXColorPalette::_DrawFocusHilite(int onoff) {
 #define CLICK_HS    1
 #define CLICK_L     2
 
-OXColorPick::OXColorPick(const OXWindow *p, int w, int h, int id = -1) :
+OXColorPick::OXColorPick(const OXWindow *p, int w, int h, int id) :
   OXFrame(p, w, h, CHILD_FRAME), OXWidget(id, "OXColorPick") {
 
   _colormapRect = ORectangle(1, 1, w - 33 - 2, h - 2);
@@ -452,9 +452,6 @@ void OXColorPick::_CreateImages() {
 // display modes with reduced number of colors. The Floyd-Steinberg error
 // diffusion dithering algorithm is used.
 
-int colormap[64][3];
-unsigned long pixel[64];
-
 // This routine is called in PseudoColor modes only.
 
 void OXColorPick::_AllocColors() {
@@ -965,7 +962,7 @@ OXColorDialog::OXColorDialog(const OXWindow *p, const OXWindow *m,
     _cpalette = new OXColorPalette(vf1, 8, 3, CDLG_CPALETTE);
     vf1->AddFrame(_cpalette, new OLayoutHints(LHINTS_NORMAL, 5, 5, 0, 0));
     _cpalette->Associate(this);
-    //_cpalette->SetColors(ucolor);
+    LoadCustomColors();
 
     // button frame
 
@@ -1080,8 +1077,8 @@ OXColorDialog::OXColorDialog(const OXWindow *p, const OXWindow *m,
     SetWMSizeHints(_w, _h, _w, _h, 0, 0);
 
     SetWindowName("Select color");
-    SetIconName("Select color");  
-    SetClassHints("xclass", "xclass");
+    SetIconName("Select color");
+    SetClassHints("XCLASS", "dialog");
 
     SetMWMHints(MWM_DECOR_ALL | MWM_DECOR_RESIZEH | MWM_DECOR_MAXIMIZE |
                                 MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
@@ -1152,7 +1149,9 @@ int OXColorDialog::ProcessMessage(OMessage *msg) {
                                   atoi(_gte->GetString()),
                                   atoi(_bte->GetString()));
               }
+              //SaveCustomColors();
             case CDLG_CANCEL:
+              SaveCustomColors();
               CloseWindow();
               break;
           }
@@ -1246,3 +1245,57 @@ int OXColorDialog::ProcessMessage(OMessage *msg) {
 
   return True;
 }
+
+void OXColorDialog::LoadCustomColors() {
+  char *inipath, line[1024], arg[PATH_MAX];
+
+  inipath = GetResourcePool()->FindIniFile("colordlg", INI_READ);
+  if (!inipath) return;
+
+  OIniFile ini(inipath, INI_READ);
+
+  while (ini.GetNext(line)) {
+    if (strcasecmp(line, "custom colors") == 0) {
+      char tmp[50];
+
+      for (int i = 0; i < 8 * 3; ++i) {
+        sprintf(tmp, "color %d", i+1);
+        if (ini.GetItem(tmp, arg)) {
+          int r, g, b;
+
+          if (sscanf(arg, "%d,%d,%d", &r, &g, &b) == 3) {
+            if (r >= 0 && r <= 255 &&
+                g >= 0 && g <= 255 &&
+                b >= 0 && b <= 255)
+              _cpalette->SetColor(i, OColor(r, g, b));
+          }
+        }
+      }
+    }
+  }
+
+  delete[] inipath;
+}
+
+void OXColorDialog::SaveCustomColors() {
+  char *inipath, tmp[20], arg[100];
+
+  inipath = GetResourcePool()->FindIniFile("colordlg", INI_WRITE);
+  if (!inipath) return;
+
+  OIniFile ini(inipath, INI_WRITE);
+
+  ini.PutNext("custom colors");
+  for (int i = 0; i < 8 * 3; ++i) {
+    OColor color = _cpalette->GetColorByIndex(i);
+    sprintf(tmp, "color %d", i+1);
+    sprintf(arg, "%d,%d,%d",
+                 color.GetR(),
+                 color.GetG(),
+                 color.GetB());
+    ini.PutItem(tmp, arg);
+  }
+  ini.PutNewLine();
+
+  delete[] inipath;
+}
