@@ -89,7 +89,7 @@ OXFileDialog::OXFileDialog(const OXWindow *p, const OXWindow *main,
 
     SetWindowName(_windowNameString);
     SetIconName(_windowNameString);
-    SetClassHints("FileDialog", "FileDialog");
+    SetClassHints("XCLASS", "dialog");
 
     SetMWMHints(MWM_DECOR_ALL | MWM_DECOR_RESIZEH | MWM_DECOR_MAXIMIZE |
                                 MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
@@ -210,7 +210,7 @@ void OXFileDialog::_FileDialog(int dlg_type, OFileInfo *file_info) {
       }
     }
     _addFavor = new OXPictureButton(_htop, paddfavor, IDF_ADDFAVOR);
-    _favor = new OXPictureButton(_htop, pfavor,IDF_FAVOR);
+    _favor = new OXPictureButton(_htop, pfavor, IDF_FAVOR);
 
     _addFavor->SetTip("Add to favourites");
     _favor->SetTip("Go to favourites folder");
@@ -355,10 +355,10 @@ void OXFileDialog::_FileDialog(int dlg_type, OFileInfo *file_info) {
   _GrabAltKey(XK_t);
 }
 
-void OXFileDialog::CloseWindow() {
+int OXFileDialog::CloseWindow() {
   if (_file_info->filename) delete[] _file_info->filename;
   _file_info->filename = NULL;
-  delete this;
+  return OXTransientFrame::CloseWindow();
 }
 
 int OXFileDialog::HandleKey(XKeyEvent *event) {
@@ -410,7 +410,7 @@ int OXFileDialog::ProcessMessage(OMessage *msg) {
   OXTextLBEntry *te;
   OFileItem *f;
   struct stat sbuf;
-  vector<OItem *> items;
+  std::vector<OItem *> items;
 
   switch (msg->type) {
   case MSG_BUTTON:
