@@ -186,7 +186,7 @@ void OXItemView::DeleteSelection() {
   for (i = 0; i < _items.size(); ++i) {
     item = _items[i];
     if (item->IsSelected()) {
-      _items.erase(&_items[i]);
+      _items.erase(_items.begin() + i);  //_items.erase(&_items[i]);
       delete item;
       --i;
     }
@@ -292,9 +292,19 @@ void OXItemView::SelectItem(OItem *item, bool state) {
 	}
     }
     item->Select(state);
+#if 0
     _hasSelection = state;
-    for (i = 0; i < _items.size(); i++)
-      _hasSelection = _hasSelection | _items[i]->IsSelected();
+    if (!state) {
+      for (i = 0; i < _items.size(); i++)
+        _hasSelection = _hasSelection | _items[i]->IsSelected();
+    }
+#else
+    if (state) {
+      _hasSelection = state;
+    } else {
+      _hasSelection = (_selectedItems.size() > 0);
+    }
+#endif
   }
 
   // can't use NeedRedraw here...
@@ -355,6 +365,8 @@ int OXItemView::HandleButton(XButtonEvent *event) {
   bool selChange = False;
   int inv, add;
 
+  if (OXView::HandleButton(event)) return True;  // wheel-mouse scroll
+
   if (event->type == ButtonPress) {
 
     if (TakesFocus()) RequestFocus();
@@ -367,13 +379,15 @@ int OXItemView::HandleButton(XButtonEvent *event) {
     _selDragStart = _selAnchor;
     _selDragEnd = _selDragStart;
 
-    if (!inv && !add && _hasSelection) {
-      selChange = True;
-      UnSelectAll();
-    }
-
     selItem = GetItemByPos(_selAnchor);
 
+    if ((event->button == Button1) || !selItem || !selItem->IsSelected()) {
+      if (!inv && !add && _hasSelection) {
+        selChange = True;
+        UnSelectAll();
+      }
+    }
+
     if (selItem) {
       if (inv) {
         if (selItem->IsSelected())
@@ -431,7 +445,7 @@ int OXItemView::HandleButton(XButtonEvent *event) {
 }
 
 int OXItemView::HandleDoubleClick(XButtonEvent *event) {
-  if (event->button != Button1) return True;
+  if (event->button != Button1) return False;
 
   if (_selectedItems.size() > 0) {
     OItemViewMessage message(_msgType, MSG_DBLCLICK, _widgetID, event->button,
@@ -611,24 +625,39 @@ int OXItemView::HandleTimer(OTimer *t) {
   return True;
 }
 
-void OXItemView::DrawRegion(OPosition coord, ODimension size, int clear) {
+int OXItemView::DrawRegion(OPosition coord, ODimension size, int clear) {
+
+  OPosition pcoord = ToPhysical(coord);
 
   // check whether the rectangle is within the visible area
 
-  if ((ToPhysical(coord).y + size.h < 0) &&
-      (ToPhysical(coord).y > _canvas->GetHeight()))
-    return;
+  if ((pcoord.y + (int) size.h < 0) || (pcoord.y > _canvas->GetHeight()))
+    return False;
 
-  if ((ToPhysical(coord).x + size.w < 0) &&
-      (ToPhysical(coord).x > _canvas->GetWidth()))
-    return;
+  if ((pcoord.x + (int) size.w < 0) || (pcoord.x > _canvas->GetWidth()))
+    return False;
 
   // clip the size argument to the canvas size
 
-  if (ToPhysical(coord).x + size.w > _canvas->GetWidth())
-    size.w = _canvas->GetWidth() - ToPhysical(coord).x;
-  if (ToPhysical(coord).y + size.h > _canvas->GetHeight())
-    size.h = _canvas->GetHeight() - ToPhysical(coord).y;
+  if (pcoord.x < 0) {
+    if (-pcoord.x > (int) size.w) return False;
+    coord.x += -pcoord.x;
+    size.w -= -pcoord.x;
+    pcoord.x = 0;
+  }
+
+  if (pcoord.y < 0) {
+    if (-pcoord.y > (int) size.h) return False;
+    coord.y += -pcoord.y;
+    size.h -= -pcoord.y;
+    pcoord.y = 0;
+  }
+
+  if (pcoord.x + (int) size.w > _canvas->GetWidth())
+    size.w = _canvas->GetWidth() - pcoord.x;
+
+  if (pcoord.y + (int) size.h > _canvas->GetHeight())
+    size.h = _canvas->GetHeight() - pcoord.y;
 
   // clear parts that are outside the object
 
@@ -636,11 +665,11 @@ void OXItemView::DrawRegion(OPosition coord, ODimension size, int clear) {
 
   // well, no items, no jobs
 
-  if (_items.size() == 0) return;
+  if (_items.size() == 0) return False;
 
   // check whether the area is inside the virtual window
 
-  if ((coord.y > _virtualSize.h) || (coord.x > _virtualSize.w)) return;
+  if ((coord.y > _virtualSize.h) || (coord.x > _virtualSize.w)) return False;
 
   // ok, at this point it is almost for sure that there has to be
   // redisplayed something
@@ -653,4 +682,6 @@ void OXItemView::DrawRegion(OPosition coord, ODimension size, int clear) {
       DrawItem(_items[i], coord, size);
     }
   }
+
+  return True;
 }
