@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Hector Peraza.
 
     This library is free software; you can redistribute it and/or
@@ -132,8 +132,7 @@ OXLBContainer::OXLBContainer(const OXWindow *p, int w, int h,
 //    SetLayoutManager(new OColumnLayout(this, 0));
 
     XGrabButton(GetDisplay(), AnyButton, AnyModifier, _id, False,
-                ButtonPressMask | ButtonReleaseMask |
-                PointerMotionMask,
+                ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                 GrabModeAsync, GrabModeAsync, None, None);
 }
 
@@ -168,9 +167,11 @@ void OXLBContainer::AddEntry(OXLBEntry *lbe, OLayoutHints *lhints) {
 }
 
 void OXLBContainer::MoveSelectUp(OXLBEntry *lbe) {
-  OXLBEntry *e,*first;
+  OXLBEntry *e, *first = NULL;
   SListFrameElt *ptr;
 
+  if (!lbe) return;
+
   //printf("OXLBContainer::MoveSelectUp(ID:%d)\n", lbe->ID());
   //if (lbe) {
   //  lbe->Activate(False);
@@ -178,9 +179,9 @@ void OXLBContainer::MoveSelectUp(OXLBEntry *lbe) {
   //}
 
   if (_flist)
-    first = (OXLBEntry *)_flist->frame;
+    first = (OXLBEntry *) _flist->frame;
 
-  if (lbe == first){
+  if (lbe == first) {
 #if 0
     ptr = _flist; 
     while (ptr->next != NULL) ptr = ptr->next;
@@ -189,8 +190,8 @@ void OXLBContainer::MoveSelectUp(OXLBEntry *lbe) {
     return;
 #endif
   } else {
-    for (ptr=_flist; ptr->next != NULL; ptr=ptr->next) {
-      if ((OXLBEntry *) ptr->next->frame== lbe) break;
+    for (ptr = _flist; ptr->next != NULL; ptr = ptr->next) {
+      if ((OXLBEntry *) ptr->next->frame == lbe) break;
     }
     e = (OXLBEntry *) ptr->frame;
   }
@@ -207,9 +208,11 @@ void OXLBContainer::MoveSelectUp(OXLBEntry *lbe) {
 }
 
 void OXLBContainer::MoveSelectDown(OXLBEntry *lbe){
-  OXLBEntry *e,*first;
+  OXLBEntry *e, *first = NULL;
   SListFrameElt *ptr;
 
+  if (!lbe) return;
+
   //if (lbe) {
   //  lbe->Activate(False);
   //  lbe->ShowFocusHilite(False);  // ==!==
@@ -217,10 +220,10 @@ void OXLBContainer::MoveSelectDown(OXLBEntry *lbe){
   //}
 
   if (_flist)
-    first = (OXLBEntry *)_flist->frame;
+    first = (OXLBEntry *) _flist->frame;
 
-  for (ptr=_flist; ptr->next != NULL; ptr=ptr->next) {
-    if ((OXLBEntry *) ptr->frame== lbe) {
+  for (ptr = _flist; ptr->next != NULL; ptr = ptr->next) {
+    if ((OXLBEntry *) ptr->frame == lbe) {
       e = (OXLBEntry *) ptr->next->frame;
       break;
     }
@@ -249,7 +252,7 @@ void OXLBContainer::InsertEntry(OXLBEntry *lbe, OLayoutHints *lhints, int afterI
 
   lbe->SetBackgroundColor(GetResourcePool()->GetDocumentBgndColor());
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
     e = (OXLBEntry *) ptr->frame;
     if (e->ID() == afterID) break;
   }
@@ -281,7 +284,7 @@ void OXLBContainer::RemoveEntry(int ID) {
   SListFrameElt *ptr;
   OLayoutHints *l;
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
     e = (OXLBEntry *) ptr->frame;
     l = ptr->layout;
     if (e->ID() == ID) {
@@ -344,13 +347,14 @@ void OXLBContainer::UnSelectAll() {
   SListFrameElt *ptr;
   OXLBEntry *f;
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
     f = (OXLBEntry *) ptr->frame;
     f->Activate(False);
     f->ShowFocusHilite(False);  // ==!==
   }
 
   _last_active = NULL;
+
   OContainerMessage message(MSG_CONTAINER, MSG_SELCHANGED, -1, 0, 0, 0);
   SendMessage(_msgObject, &message);
 }
@@ -362,7 +366,7 @@ void OXLBContainer::SelectAll() {
 
   if (!_multipleSelect) return;
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
     f = (OXLBEntry *) ptr->frame;
     f->Activate(True);
     ++total;
@@ -384,7 +388,7 @@ OXLBEntry *OXLBContainer::Select(int ID) {
     _last_active = NULL;
   }
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
     f = (OXLBEntry *) ptr->frame;
     if (f->ID() == ID) {
       f->Activate(True);
@@ -403,7 +407,7 @@ OXLBEntry *OXLBContainer::FindEntry(int ID) {
   SListFrameElt *ptr;
   OXLBEntry *e;
 
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
     e = (OXLBEntry *) ptr->frame;
     if (e->ID() == ID) return e;
   }
@@ -411,6 +415,13 @@ OXLBEntry *OXLBContainer::FindEntry(int ID) {
   return NULL;
 }
 
+OXLBEntry *OXLBContainer::GetFirstEntry() {
+  if (_flist) 
+    return (OXLBEntry *) _flist->frame;
+  else
+    return NULL;
+}
+
 int OXLBContainer::GetSelected() const {
   if (_last_active == NULL) return -1;
   return _last_active->ID();
@@ -425,7 +436,7 @@ int OXLBContainer::HandleKey(XKeyEvent *event) {
 
     if (keysym == XK_Down) {
       if (_last_active) {
-        for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+        for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
           f = (OXLBEntry *) ptr->frame;
           if (f == _last_active) {
             if (ptr->next) f = (OXLBEntry *) ptr->next->frame;
@@ -438,7 +449,7 @@ int OXLBContainer::HandleKey(XKeyEvent *event) {
 
     } else if (keysym == XK_Up) {
       if (_last_active) {
-        for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+        for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
           f = (OXLBEntry *) ptr->frame;
           if (f == _last_active) {
             if (ptr->prev) f = (OXLBEntry *) ptr->prev->frame;
@@ -514,6 +525,12 @@ int OXLBContainer::HandleButton(XButtonEvent *event) {
   int selected = 0;
 
   if (event->type == ButtonPress) {
+    if (event->button == Button4) {
+      return True;
+    } else if (event->button == Button5) {
+      return True;
+    }
+
     if (event->state & ShiftMask) {
       if (_multipleSelect) {
         if (_last_active) {
@@ -521,7 +538,7 @@ int OXLBContainer::HandleButton(XButtonEvent *event) {
           // and this.
           int highlighting = False;
 
-          for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+          for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
             f = (OXLBEntry *) ptr->frame;
             if ((f->GetId() == event->subwindow) ||
                  (f == _last_active)) {
@@ -540,7 +557,7 @@ int OXLBContainer::HandleButton(XButtonEvent *event) {
     } else if (event->state & ControlMask) {
       if (_multipleSelect) {
         // toggle this item
-        for (ptr=_flist; ptr != NULL; ptr=ptr->next) {
+        for (ptr = _flist; ptr != NULL; ptr = ptr->next) {
           f = (OXLBEntry *) ptr->frame;
           if (f->GetId() == event->subwindow) {
             f->Activate(!f->IsActive());
@@ -560,7 +577,7 @@ int OXLBContainer::HandleButton(XButtonEvent *event) {
         f->ShowFocusHilite(False);
         _last_active = NULL;
       }
-      for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+      for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
         f = (OXLBEntry *) ptr->frame;
         if (f->GetId() == event->subwindow) {
           f->Activate(True);
@@ -604,7 +621,7 @@ int OXLBContainer::HandleMotion(XMotionEvent *event) {
 //    f->Activate(False);
 //    _last_active = NULL;
 //  }
-  for (ptr=_flist; ptr != NULL; ptr=ptr->next) {   
+  for (ptr = _flist; ptr != NULL; ptr = ptr->next) {   
     f = (OXLBEntry *) ptr->frame;
     if (f->GetId() == event->subwindow) {
       f->Activate(True);
@@ -718,27 +735,30 @@ void OXListBox::Update() {
 }
 
 void OXListBox::Resize(int w, int h) {
-  if (_integralHeight) 
-    h = max(_itemVsize, ((h - (_bw << 1)) / _itemVsize) * _itemVsize) 
-        + (_bw << 1);
+  if (_integralHeight) {
+    int ih = _insets.t + _insets.b;
+    h = max(_itemVsize, ((h - ih) / _itemVsize) * _itemVsize) + ih;
+  }
   OXCompositeFrame::Resize(w, h);
 }
 
 void OXListBox::MoveResize(int x, int y, int w, int h) {
-  if (_integralHeight) 
-    h = max(_itemVsize, ((h - (_bw << 1)) / _itemVsize) * _itemVsize)
-        + (_bw << 1);
+  if (_integralHeight) {
+    int ih = _insets.t + _insets.b;
+    h = max(_itemVsize, ((h - ih) / _itemVsize) * _itemVsize) + ih;
+  }
   OXCompositeFrame::MoveResize(x, y, w, h);
 }
 
 ODimension OXListBox::GetDefaultSize() const {
   int h;
 
-  if (_integralHeight)
-    h = max(_itemVsize, ((_h - (_bw << 1)) / _itemVsize) * _itemVsize)
-        + (_bw << 1);
-  else
+  if (_integralHeight) {
+    int ih = _insets.t + _insets.b;
+    h = max(_itemVsize, ((_h - ih) / _itemVsize) * _itemVsize) + ih;
+  } else {
     h = _h;
+  }
 
   return ODimension(_w, h);
 }
@@ -768,19 +788,19 @@ void OXListBox::Layout() {
 
   // test whether we need vertical scrollbar or not
 
-  cw = _w-(_bw << 1);
-  ch = _h-(_bw << 1);
+  cw = _w - _insets.l - _insets.r;
+  ch = _h - _insets.t - _insets.b;
 
   _container->SetWidth(cw);
   _container->SetHeight(ch);
 
   if (_container->GetDefaultHeight() > ch) {
     need_vsb = True;
-    cw = _w-(_bw << 1)-_vscrollbar->GetDefaultWidth();
+    cw = _w - _insets.l - _insets.r - _vscrollbar->GetDefaultWidth();
     _container->SetWidth(cw);
   }
 
-  _vport->MoveResize(_bw, _bw, cw, ch);
+  _vport->MoveResize(_insets.l, _insets.t, cw, ch);
   _container->Layout();
   tch = max(_container->GetDefaultHeight(), ch);
   _container->SetHeight(0); // force a resize in OXFrame::Resize
@@ -788,7 +808,8 @@ void OXListBox::Layout() {
 //  _vport->SetPos(0, 0);
 
   if (need_vsb) {
-    _vscrollbar->MoveResize(cw+_bw, _bw, _vscrollbar->GetDefaultWidth(), ch);
+    _vscrollbar->MoveResize(cw + _insets.l, _insets.t,
+                            _vscrollbar->GetDefaultWidth(), ch);
     _vscrollbar->MapWindow();
   } else {
     _vscrollbar->UnmapWindow();
