@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Hector Peraza.
 
     This code is partially based on Robert W. McMullen's ListTree-3.0
@@ -41,14 +41,13 @@
 
 // TODO:
 // - add support for drag and drop
-// - optimize redrawings, get rid of __NeedFullRedraw()
 // - send more descriptive messages to the application
 // - implement a new OXFileSystemTree class
 
 
 //--------------------------------------------------------------------
 
-OListTreeItem::OListTreeItem(OXClient *client, char *name, 
+OListTreeItem::OListTreeItem(OXClient *client, const char *name, 
                              const OPicture *opened,
                              const OPicture *closed) {
   length = strlen(name);
@@ -76,7 +75,7 @@ OListTreeItem::~OListTreeItem() {
   _client->FreePicture(closed_pic);
 }
 
-void OListTreeItem::Rename(char *new_name) {
+void OListTreeItem::Rename(const char *new_name) {
   delete[] text;
   length = strlen(new_name);
   text = new char[length + 1];
@@ -186,7 +185,10 @@ void OXListTree::_HighlightItem(OListTreeItem *item, int state, int draw) {
     if ((item == _selected) && !state) _selected = NULL;
     if (state != item->active) {
       item->active = state;
-      if (draw) _DrawItemName(item);
+      if (draw) {
+        _DrawItemPic(item);
+        _DrawItemName(item);
+      }
     }
   }
 }
@@ -204,13 +206,11 @@ void OXListTree::_UnselectAll(int draw) {
   _HighlightChildren(_first, False, draw);
 }
 
-void OXListTree::__NeedFullRedraw(int clrbg) {
-  NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
-}
-
 int OXListTree::HandleButton(XButtonEvent *event) {
   OListTreeItem *item;
 
+  if (OXView::HandleButton(event)) return True;  // mouse-wheel scroll
+
   OPosition ep = ToVirtual(OPosition(event->x, event->y));
 
   if (event->type == ButtonPress) {
@@ -221,16 +221,13 @@ int OXListTree::HandleButton(XButtonEvent *event) {
         if ((ep.x > item->xnode-5) && (ep.x < item->xnode+5)) {
           // toggle open 
           item->open = !item->open;
-          __NeedFullRedraw(True);
+          NeedRedraw(ORectangle(0, item->y,
+                     _virtualSize.w, _canvas->GetHeight()));
         }
       }
-      if (_selected) _selected->active = False;
       _last_y = ep.y;
       _UnselectAll(True);
-      _selected = item;
-      //item->active = True; // this is done below w/redraw
-      _HighlightItem(item, True, True);
-      __NeedFullRedraw(False);
+      _HighlightItem(_selected = item, True, True);
       OListTreeMessage msg(_msgType, MSG_CLICK, _widgetID,
                            event->button, event->x_root, event->y_root);
       SendMessage(_msgObject, &msg);
@@ -242,6 +239,8 @@ int OXListTree::HandleButton(XButtonEvent *event) {
 int OXListTree::HandleDoubleClick(XButtonEvent *event) {
   OListTreeItem *item;
 
+  if (event->button != Button1) return False;
+
   OPosition ep = ToVirtual(OPosition(event->x, event->y));
 
   if ((item = _FindItem(ep.y)) != NULL) {
@@ -251,12 +250,11 @@ int OXListTree::HandleDoubleClick(XButtonEvent *event) {
       }
     }
     item->open = !item->open;
-    if (_selected) _selected->active = False;
+    NeedRedraw(ORectangle(0, item->y,
+               _virtualSize.w, _canvas->GetHeight()));
     _UnselectAll(True);
     _selected = item;
-    //item->active = True; // this is done below w/redraw
-    _HighlightItem(item, True, True);
-    __NeedFullRedraw(99);
+    _HighlightItem(_selected = item, True, True);
     OListTreeMessage msg(_msgType, MSG_DBLCLICK, _widgetID,
                          event->button, event->x_root, event->y_root);
     SendMessage(_msgObject, &msg);
@@ -265,9 +263,10 @@ int OXListTree::HandleDoubleClick(XButtonEvent *event) {
   return True;
 }
 
-void OXListTree::DrawRegion(OPosition coord, ODimension size, int clear) {
+int OXListTree::DrawRegion(OPosition coord, ODimension size, int clear) {
   OXView::DrawRegion(coord, size, clear);
   _Draw(coord.y, size.h);
+  return True;
 }
 
 int OXListTree::HandleKey(XKeyEvent *event) {
@@ -276,7 +275,6 @@ int OXListTree::HandleKey(XKeyEvent *event) {
   if (!_selected) {
     _selected = _first;
     _HighlightItem(_selected, True, True);
-    __NeedFullRedraw(99);
     return True;
   }
 
@@ -291,6 +289,8 @@ int OXListTree::HandleKey(XKeyEvent *event) {
         // otherwise go to parent node
         if (_selected->open) {
           _selected->open = False;
+          NeedRedraw(ORectangle(0, _selected->y,
+                     _virtualSize.w, _canvas->GetHeight()));
         } else if (_selected != _first) {
           next = _selected->parent;
           if (_timer) delete _timer;
@@ -302,7 +302,11 @@ int OXListTree::HandleKey(XKeyEvent *event) {
         // if the current node is closed, open it
         // otherwise go to the first child
         if (!_selected->open) {
-          if (_selected->firstchild) _selected->open = True;
+          if (_selected->firstchild) {
+            _selected->open = True;
+            NeedRedraw(ORectangle(0, _selected->y,
+                       _virtualSize.w, _canvas->GetHeight()));
+          }
         } else if (_selected->firstchild) {
           next = _selected->firstchild;
           if (_timer) delete _timer;
@@ -380,6 +384,8 @@ int OXListTree::HandleKey(XKeyEvent *event) {
       case XK_KP_Add:
         // expand one level
         _selected->open = True;
+        NeedRedraw(ORectangle(0, _selected->y,
+                   _virtualSize.w, _canvas->GetHeight()));
         msg.action = MSG_DBLCLICK;
         SendMessage(_msgObject, &msg);
         break;
@@ -388,6 +394,8 @@ int OXListTree::HandleKey(XKeyEvent *event) {
       case XK_KP_Subtract:
         // collapse one level
         _selected->open = False;
+        NeedRedraw(ORectangle(0, _selected->y,
+                   _virtualSize.w, _canvas->GetHeight()));
         msg.action = MSG_DBLCLICK;
         SendMessage(_msgObject, &msg);
         break;
@@ -436,12 +444,11 @@ int OXListTree::HandleKey(XKeyEvent *event) {
     }
 
     if (next != _selected) {
-      _selected->active = False;
+      _HighlightItem(_selected, False, True);
       _selected = next;
       _EnsureVisible(_selected);
     }
     _HighlightItem(_selected, True, True);
-    __NeedFullRedraw(99);
   }
 
   return True;
@@ -479,7 +486,7 @@ void OXListTree::_LostFocus() {
 void OXListTree::ShowFocusHilite(int onoff) {
   if (_selected) {
     NeedRedraw(ORectangle(0, _selected->y,
-               _canvas->GetWidth(), _selected->height));
+               _virtualSize.w, _selected->height));
   }
 }
 
@@ -906,7 +913,30 @@ OListTreeItem *OXListTree::_FindItem(int findy) {
 
 //----------------------------------------------- Public Functions
 
-OListTreeItem *OXListTree::AddItem(OListTreeItem *parent, char *string,
+void OXListTree::AddItem(OListTreeItem *parent, OListTreeItem *item) {
+
+  if (!item->open_pic)
+    item->open_pic = _client->GetPicture("ofolder.t.xpm");
+  if (!item->closed_pic)
+    item->closed_pic = _client->GetPicture("folder.t.xpm");
+
+  _InsertChild(parent, item);
+
+  int affects_display = True;
+
+  while (parent) {
+    if (!parent->open) {
+      affects_display = False;
+      break;
+    }
+    parent = parent->parent;
+  }
+
+  if (affects_display)
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
+}
+
+OListTreeItem *OXListTree::AddItem(OListTreeItem *parent, const char *string,
                                    const OPicture *open,
                                    const OPicture *closed) {
   OListTreeItem *item;
@@ -917,17 +947,30 @@ OListTreeItem *OXListTree::AddItem(OListTreeItem *parent, char *string,
   item = new OListTreeItem(_client, string, open, closed);
   _InsertChild(parent, item);
 
-  __NeedFullRedraw(99);
+  int affects_display = True;
+
+  while (parent) {
+    if (!parent->open) {
+      affects_display = False;
+      break;
+    }
+    parent = parent->parent;
+  }
+
+  if (affects_display)
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
 
   return item;
 }
 
-void OXListTree::RenameItem(OListTreeItem *item, char *string) {
+void OXListTree::RenameItem(OListTreeItem *item, const char *string) {
   item->Rename(string);
-  __NeedFullRedraw(99);
+  NeedRedraw(ORectangle(0, item->y, _virtualSize.w, item->height));
 }
 
 int OXListTree::DeleteItem(OListTreeItem *item) {
+  int y = item->y;
+  OListTreeItem *parent = item->parent;
 
   if (item->firstchild)
     _DeleteChildren(item->firstchild);
@@ -936,19 +979,41 @@ int OXListTree::DeleteItem(OListTreeItem *item) {
   _RemoveReference(item);
   delete item;
 
-  __NeedFullRedraw(99);
+  int affects_display = True;
+
+  while (parent) {
+    if (!parent->open) {
+      affects_display = False;
+      break;
+    }
+    parent = parent->parent;
+  }
+
+  if (affects_display)
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
 
   return 1;
 }
 
 int OXListTree::DeleteChildren(OListTreeItem *item) {
+  int y = item->y;
 
   if (item->firstchild)
     _DeleteChildren(item->firstchild);
 
   item->firstchild = NULL;
 
-  __NeedFullRedraw(99);
+  int affects_display = True;
+
+  for (item = item->parent; item; item = item->parent) {
+    if (!item->open) {
+      affects_display = False;
+      break;
+    }
+  }
+
+  if (affects_display)
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
 
   return 1;
 }
@@ -961,7 +1026,7 @@ int OXListTree::Reparent(OListTreeItem *item, OListTreeItem *newparent) {
   // The item is now unattached. Reparent it.
   _InsertChild(newparent, item);
 
-  __NeedFullRedraw(99);
+  NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
 
   return 1;
 }
@@ -976,18 +1041,36 @@ int OXListTree::ReparentChildren(OListTreeItem *item,
 
     _InsertChildren(newparent, first);
 
-    __NeedFullRedraw(99);
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
     return 1;
   }
   return 0;
 }
 
+void OXListTree::OpenNode(OListTreeItem *item) {
+  if (!item->open) {
+    item->open = True;
+    NeedRedraw(ORectangle(0, item->y,
+               _virtualSize.w, _canvas->GetHeight()));
+  }
+}
+
+void OXListTree::CloseNode(OListTreeItem *item) {
+  if (item->open) {
+    item->open = False;
+    NeedRedraw(ORectangle(0, item->y,
+               _virtualSize.w, _canvas->GetHeight()));
+  }
+}
+
 //static int OXListTree::_compare(const void *item1, const void *item2) {
 int _compare(const void *item1, const void *item2) {
   return strcmp((*((OListTreeItem **) item1))->text,
 		(*((OListTreeItem **) item2))->text);
 }
 
+// Sort siblings
+
 int OXListTree::Sort(OListTreeItem *item) {
   OListTreeItem *first, *parent, **list;
   size_t i, count;
@@ -996,20 +1079,29 @@ int OXListTree::Sort(OListTreeItem *item) {
   while (item->prevsibling) item = item->prevsibling;
 
   first = item;
+
+  i = 0;
+  while (item->nextsibling) {
+    if (strcmp(item->text, item->nextsibling->text) > 0) { i = 1; break; }
+    item = item->nextsibling;
+  }
+
+  if (i == 0) return 1;  // no need to sort!
+
   parent = first->parent;
 
   // Count the children
   count = 1;
-  while (item->nextsibling) item = item->nextsibling, count++;
+  for (item = first ; item->nextsibling ; item = item->nextsibling) count++;
   if (count <= 1) return 1;
 
   list = new OListTreeItem* [count];
-  list[0] = first;
+  list[0] = item = first;
   count = 1;
-  while (first->nextsibling) {
-    list[count] = first->nextsibling;
+  while (item->nextsibling) {
+    list[count] = item->nextsibling;
     count++;
-    first = first->nextsibling;
+    item = item->nextsibling;
   }
 
   qsort(list, count, sizeof(OListTreeItem*), _compare);
@@ -1029,7 +1121,17 @@ int OXListTree::Sort(OListTreeItem *item) {
 
   delete[] list;
 
-  __NeedFullRedraw(99);
+  int affects_display = True;
+
+  for (item = first->parent; item; item = item->parent) {
+    if (!item->open) {
+      affects_display = False;
+      break;
+    }
+  }
+
+  if (affects_display)
+    NeedRedraw(ORectangle(_visibleStart, _canvas->GetSize()));
 
   return 1;
 }
@@ -1051,7 +1153,8 @@ int OXListTree::SortChildren(OListTreeItem *item) {
   return 1;
 }
 
-OListTreeItem *OXListTree::FindSiblingByName(OListTreeItem *item, char *name) {
+OListTreeItem *OXListTree::FindSiblingByName(OListTreeItem *item,
+                                             const char *name) {
   OListTreeItem *first;
 
   // Get first child in list;
@@ -1070,7 +1173,8 @@ OListTreeItem *OXListTree::FindSiblingByName(OListTreeItem *item, char *name) {
   return NULL;
 }
 
-OListTreeItem *OXListTree::FindChildByName(OListTreeItem *item, char *name) {
+OListTreeItem *OXListTree::FindChildByName(OListTreeItem *item,
+                                           const char *name) {
 
   // Get first child in list
   if (item && item->firstchild) {
@@ -1090,15 +1194,13 @@ OListTreeItem *OXListTree::FindChildByName(OListTreeItem *item, char *name) {
 }
 
 void OXListTree::HighlightItem(OListTreeItem *item, int ensure_visible) {
-  _UnselectAll(False);
-  _HighlightItem(_selected = item, True, False);
+  _UnselectAll(True);
+  _HighlightItem(_selected = item, True, True);
   if (ensure_visible) _EnsureVisible(item);
-  __NeedFullRedraw(False);
 }
 
 void OXListTree::ClearHighlighted() {
-  _UnselectAll(False);
-  __NeedFullRedraw(False);
+  _UnselectAll(True);
 }
 
 void OXListTree::EnsureVisible(OListTreeItem *item) {
