@@ -19,8 +19,9 @@
 
 **************************************************************************/
 
-#include <vector.h>
-#include <algo.h>
+#include <vector>
+#include <algorithm>
+#include <functional>
 
 #include <X11/keysym.h>
 
@@ -39,7 +40,7 @@ const OXGC *OListViewItem::_dfnormGC;
 
 OListViewItem::OListViewItem(const OXListView *p, int id,
          const OPicture *bigpic, const OPicture *smallpic,
-         vector<OString *> names, int viewMode) :
+         std::vector<OString *> names, int viewMode) :
   OItem(p, id) {
 
   if (!_defaultFont) {
@@ -198,7 +199,7 @@ void OListViewItem::Draw(OXWindow *w, OPosition pos) {
     pic = _currentPic;
   }
 
-  vector<SColumnData *> columnData = ((OXListView *)_parent)->GetColumns();
+  std::vector<SColumnData *> columnData = ((OXListView *)_parent)->GetColumns();
 
   // displaying icon and name
 
@@ -367,6 +368,20 @@ OXListView::~OXListView() {
   delete _header;
 }
 
+int OXListView::HandleButton(XButtonEvent *event) {
+
+  if ((_viewMode == LV_LIST) && (event->type == ButtonPress)) {
+    if (event->button == Button4) {
+      ScrollRight(_scrollValue.x);
+      return True;
+    } else if (event->button == Button5) {
+      ScrollLeft(_scrollValue.x);
+      return True;
+    }
+  }
+
+  return OXItemView::HandleButton(event);
+}
 
 //----------------------------------------------------------------------
 
@@ -475,10 +490,14 @@ int OXListView::HandleKey(XKeyEvent *event) {
             _anchorItem = item;
             ShowFocusHilite(HasFocus());
           }
+        } else {
+          return True;  // probably a modifier key...
         }
         }
         break;
     }
+  } else {
+    return False;  // key release
   }
 
   if (_anchorItem) {
@@ -713,14 +732,20 @@ void OXListView::ShowHeaderBar() {
   int n = _columnData.size();
 
   if (n > 0) {
-    int i, w = 0, h = 0, xl = -_visibleStart.x, bfw = _w - 2*_bw;
+    int i, w, h, xl, bfw;
+
+    w = h = 0;
+
+    xl = -_visibleStart.x;
+    bfw = _w - _insets.r - _insets.l;
 
     h = _columnData[0]->button->GetDefaultHeight() - 4;
     for (i = 0; i < n; ++i) {
       w = _columnData[i]->width;
       if (i == 0) w += _itemSep.w;
       if (i == n - 1) w = bfw - xl;
-      _columnData[i]->resizer->MoveResize(xl+w-1+_bw, _bw, 2, _h-2*_bw);
+      _columnData[i]->resizer->MoveResize(xl + w - 1 + _insets.l, _insets.t,
+                                          2, _h - _insets.t - _insets.b);
       _columnData[i]->resizer->MoveResizeHandle(-3, 0, 6, h);
       if ((xl + w > 0) && (xl < bfw)) {
         _columnData[i]->button->MoveResize(xl, 0, w, h);
@@ -735,7 +760,7 @@ void OXListView::ShowHeaderBar() {
       }
       xl += w;
     }
-    _header->MoveResize(_bw, _bw, bfw, h);
+    _header->MoveResize(_insets.l, _insets.t, bfw, h);
     _header->MapWindow();
   }
 }
@@ -807,6 +832,9 @@ void OXListView::AddItem(OListViewItem *newItem) {
   const OResourcePool *res = _client->GetResourcePool();
   if (_itemFont) newItem->SetFont(res->GetFontPool()->GetFont(_itemFont));
 
+  ODimension itemSize = newItem->GetDefaultSize();
+  newItem->Resize(itemSize);
+
   _items.push_back(newItem);
 
   // Adjust the column widths for LV_DETAILS, if necessary
@@ -832,6 +860,7 @@ void OXListView::AddItem(OListViewItem *newItem) {
 
   newItem->SetViewMode(_viewMode);
   CalcMaxItemSize();
+
 //  Layout();
 }
 
@@ -921,7 +950,7 @@ bool OXListView::ItemLayout() {
 
         itemSize = _items[i]->GetDefaultSize();
 
-        itemPos.x = currentPos.x + (maxItemSize.w - itemSize.w) / 2;
+        itemPos.x = currentPos.x + ((int) maxItemSize.w - (int) itemSize.w) / 2;
         itemPos.y = currentPos.y;
 
         if ((_items[i]->GetPosition().x != itemPos.x) ||
@@ -956,7 +985,7 @@ bool OXListView::ItemLayout() {
         itemSize = _items[i]->GetDefaultSize();
 
         itemPos.x = currentPos.x;
-        itemPos.y = currentPos.y + (maxItemSize.h - itemSize.h) / 2;
+        itemPos.y = currentPos.y + ((int) maxItemSize.h - (int) itemSize.h) / 2;
 
         if ((_items[i]->GetPosition().x != itemPos.x) ||
             (_items[i]->GetPosition().y != itemPos.y)) {
@@ -990,7 +1019,7 @@ bool OXListView::ItemLayout() {
         itemSize = _items[i]->GetDefaultSize();
 
         itemPos.x = currentPos.x;
-        itemPos.y = currentPos.y + (maxItemSize.h - itemSize.h) / 2;
+        itemPos.y = currentPos.y + ((int) maxItemSize.h - (int) itemSize.h) / 2;
 
         if ((_items[i]->GetPosition().x != itemPos.x) ||
             (_items[i]->GetPosition().y != itemPos.y)) {
@@ -1036,10 +1065,36 @@ bool OXListView::ItemLayout() {
   return needRedraw;
 }
 
-void OXListView::DrawRegion(OPosition coord, ODimension size, int clear) {
-  OXItemView::DrawRegion(coord, size, clear);
+int OXListView::DrawRegion(OPosition coord, ODimension size, int clear) {
 
-  if ((coord.y > _virtualSize.h) || (coord.x > _virtualSize.w)) return;
+  if (!OXItemView::DrawRegion(coord, size, clear)) return False;
+
+#if 1
+  OPosition pcoord = ToPhysical(coord);
+
+  if (pcoord.x < 0) {
+    if (-pcoord.x > (int) size.w) return False;
+    coord.x += -pcoord.x;
+    size.w -= -pcoord.x;
+    pcoord.x = 0;
+  }
+
+  if (pcoord.y < 0) {
+    if (-pcoord.y > (int) size.h) return False;
+    coord.y += -pcoord.y;
+    size.h -= -pcoord.y;
+    pcoord.y = 0;
+  }
+
+  if (pcoord.x + (int) size.w > _canvas->GetWidth())
+    size.w = _canvas->GetWidth() - pcoord.x;
+
+  if (pcoord.y + (int) size.h > _canvas->GetHeight())
+    size.h = _canvas->GetHeight() - pcoord.y;
+
+  // check whether the area is inside the virtual window
+
+  if ((coord.y > _virtualSize.h) || (coord.x > _virtualSize.w)) return False;
 
   if (_viewMode == LV_DETAILS) {
     for (unsigned int i = 0; i < _items.size(); ++i) {
@@ -1048,9 +1103,12 @@ void OXListView::DrawRegion(OPosition coord, ODimension size, int clear) {
         DrawItem(_items[i], coord, size);
     }
   }
+#endif
+
+  return True;
 }
 
-struct SLVSortAscending : public binary_function<OItem*, OItem*, bool> {
+struct SLVSortAscending : public std::binary_function<OItem*, OItem*, bool> {
 public:
   bool operator()(const OItem* item1, const OItem* item2) const {
     OListViewItem *i1 = (OListViewItem *) item1;
@@ -1062,7 +1120,7 @@ public:
   }
 };
 
-struct SLVSortDescending : public binary_function<OItem*, OItem*, bool> {
+struct SLVSortDescending : public std::binary_function<OItem*, OItem*, bool> {
 public:
   bool operator()(const OItem* item1, const OItem* item2) const {
     OListViewItem *i1 = (OListViewItem *) item1;
