@@ -1,6 +1,6 @@
 /**************************************************************************
 
-    This file is part of Xclass95, a Win95-looking GUI toolkit.
+    This file is part of xclass, a Win95-looking GUI toolkit.
     Copyright (C) 1996, 1997 David Barth, Hector Peraza.
 
     This library is free software; you can redistribute it and/or
@@ -99,12 +99,8 @@ void OXMenuBar::AddPopup(OHotString *s, OXPopupMenu *menu, OLayoutHints *l) {
 int OXMenuBar::HandleButton(XButtonEvent *event) {
   SListFrameElt *ptr;
 
-//// OnyX
-////      if (_current && _current->HandleButton(event)) return True;
-//// OnyX
-
   // Here we are looking for a _child_ of a menu bar, so it can _ONLY_ be
-  // a OXMenuTitle : BEWARE !
+  // a OXMenuTitle: BEWARE!
 
   OXMenuTitle *target;
   
@@ -113,31 +109,29 @@ int OXMenuBar::HandleButton(XButtonEvent *event) {
 
   if (event->type == ButtonPress) {
 
-//// OnyX
     if (_current && _current->HandleButton(event)) return True;
-//// OnyX
 
-    target = (OXMenuTitle*)_client->GetWindowById(event->subwindow);
+    target = (OXMenuTitle *) _client->GetWindowById(event->subwindow);
 
     _x0 = event->x;
     _y0 = event->y;
 
-    if (target != NULL) {
+    if (target) {
       _stick = True;
 
       if (target != _current) {
         // Deactivate all others :
         for (ptr = _flist; ptr != NULL; ptr = ptr->next)
-          ((OXMenuTitle*)ptr->frame)->SetState(False);
+          ((OXMenuTitle *)ptr->frame)->SetState(False);
 
         target->SetState(True);
         _current = target;
 
-        XGrabPointer(GetDisplay(), _id, False, //// <-OnyX Normal-> True,
+        XGrabPointer(GetDisplay(), _id, False,
                      ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                      GrabModeAsync, GrabModeAsync,
                      None, GetResourcePool()->GetGrabCursor(), CurrentTime);
-        XGrabKeyboard(GetDisplay(), _id, False/*True*/,
+        XGrabKeyboard(GetDisplay(), _id, False,
                       GrabModeAsync, GrabModeAsync,
                       CurrentTime);
       }
@@ -146,23 +140,19 @@ int OXMenuBar::HandleButton(XButtonEvent *event) {
   
   if (event->type == ButtonRelease) {
 
-//// OnyX
-////    if (_current && _current->HandleButton(event)) /*return True*/;
-//// OnyX
-
     if (_stick) {
       _stick = False;
       return True;
     }
     XUngrabPointer(GetDisplay(), CurrentTime);
     XUngrabKeyboard(GetDisplay(), CurrentTime);
-    XFlush(GetDisplay());
+    XSync(GetDisplay(), False);
 
     for (ptr = _flist; ptr != NULL; ptr = ptr->next)
-      ((OXMenuTitle*)ptr->frame)->SetState(False);
+      ((OXMenuTitle *)ptr->frame)->SetState(False);
 
-    if (_current != NULL) {
-      target = _current; // tricky, because WaitFor
+    if (_current) {
+      target = _current; // tricky, because WaitFor()
       _current = NULL;
       target->DoSendMessage();
     }
@@ -184,18 +174,16 @@ int OXMenuBar::HandleMotion(XMotionEvent *event) {
 
   _stick = False;
 
-//// OnyX
-      if (_current && _current->HandleMotion(event)) return True;
-//// OnyX
+  if (_current && _current->HandleMotion(event)) return True;
 
   XTranslateCoordinates(GetDisplay(), _id, _id, event->x, event->y,
                         &dummy, &dummy, &wtarget);
-  target = (OXMenuTitle*)_client->GetWindowById(wtarget);
+  target = (OXMenuTitle *)_client->GetWindowById(wtarget);
 
-  if (target != NULL && target != _current) {
+  if (target && target != _current) {
     // Deactivate all others :
     for (ptr = _flist; ptr != NULL; ptr = ptr->next)
-      ((OXMenuTitle*)ptr->frame)->SetState(False);
+      ((OXMenuTitle *)ptr->frame)->SetState(False);
 
     target->SetState(True);
     _stick = True;
@@ -267,7 +255,7 @@ int OXMenuBar::HandleKey(XKeyEvent *event) {
       if (target != _current) {
         // Deactivate all others :
         for (ptr = _flist; ptr != NULL; ptr = ptr->next)
-          ((OXMenuTitle*)ptr->frame)->SetState(False);
+          ((OXMenuTitle *)ptr->frame)->SetState(False);
 
         target->SetState(True);
         _stick = True;
@@ -277,12 +265,12 @@ int OXMenuBar::HandleKey(XKeyEvent *event) {
                      ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                      GrabModeAsync, GrabModeAsync,
                      None, GetResourcePool()->GetGrabCursor(), CurrentTime);
-/* Because of the way the key binding mechanism currently works, we have
-   to grab the keyboard _after_ releasing the first key!
-        XGrabKeyboard(GetDisplay(), _id, True,
-                      GrabModeAsync, GrabModeAsync,
-                      CurrentTime);
-*/
+
+        // Because of the way the OXMainFrame key binding mechanism
+        // currently works, we can't grab the keyboard right here.
+        // We'll do it instead after the first key has been released
+        // (see below).
+
       }
     }
   }
@@ -290,9 +278,10 @@ int OXMenuBar::HandleKey(XKeyEvent *event) {
   if (event->type == KeyRelease) {
     if (_stick) {
       _stick = False;
-      // grab the keyboard HERE!
 
-      XGrabKeyboard(GetDisplay(), _id, False/*True*/,
+      // Grab the keyboard HERE!
+
+      XGrabKeyboard(GetDisplay(), _id, False,
                     GrabModeAsync, GrabModeAsync,
                     CurrentTime);
       return True;
@@ -302,13 +291,13 @@ int OXMenuBar::HandleKey(XKeyEvent *event) {
 
     XUngrabPointer(GetDisplay(), CurrentTime);
     XUngrabKeyboard(GetDisplay(), CurrentTime);
-    XFlush(GetDisplay());
+    XSync(GetDisplay(), False);
 
     for (ptr = _flist; ptr != NULL; ptr = ptr->next)
-      ((OXMenuTitle*)ptr->frame)->SetState(False);
+      ((OXMenuTitle *)ptr->frame)->SetState(False);
 
     if (_current) {
-      target = _current; // tricky, because WaitFor
+      target = _current; // tricky, because WaitFor()
       _current = NULL;
       target->DoSendMessage();
     }
@@ -317,9 +306,50 @@ int OXMenuBar::HandleKey(XKeyEvent *event) {
   return True;
 }
 
+
 //-----------------------------------------------------------------
 
-OXPopupMenu::OXPopupMenu(const OXWindow *p, int w, int h,
+OMenuEntry::OMenuEntry(int type, int id, OHotString *s, const OPicture *p) {
+  _type = type;
+  _entryID = id;
+  _label = s;
+  _pic = p;
+  _popup = NULL;
+  _status = MENU_ENABLE_MASK;
+  _hkeycode = 0;
+  prev = next = NULL;
+}
+
+OMenuEntry::~OMenuEntry() {
+  if (_label) delete _label;
+}
+
+
+//-----------------------------------------------------------------
+
+// space for the radio- and checkmarks:
+#define CHECKMARK_WIDTH  8
+
+// where the label starts (left margin):
+#define LEFT_GAP         (CHECKMARK_WIDTH + 8)
+
+// space between the icon and the text:
+#define PIC_GAP          6
+
+// width of the placeholder for the icon:
+#define PICHOLDER_WIDTH  (PIC_GAP + LEFT_GAP)   
+
+// space between left and right labels:
+#define TEXT_GAP         10
+
+// right margin:
+#define RIGHT_GAP        14
+
+// width of the popup menu symbol:
+#define POPUPMARK_WIDTH  8
+
+
+OXPopupMenu::OXPopupMenu(const OXWindow *p, int w, int h, 
 			 unsigned int options)
   : OXFrame(p, w, h, options | OWN_BKGND ) {
     XSetWindowAttributes wattr;
@@ -345,13 +375,20 @@ OXPopupMenu::OXPopupMenu(const OXWindow *p, int w, int h,
     _delay = NULL;
 
     _bw = 3;
-    _xl = _xr = 16;
+    _xl = _xr = LEFT_GAP;
     _first = _last = NULL;
+
+    _sidePic = NULL;
+    _sideBgnd = _backPixel;
+
     _msgObject = p;
     _stick = True;
     _current = NULL;
     _hasgrab = False;
     _popdown = False;
+
+    _lastID = -1;
+
     mask = CWOverrideRedirect | CWSaveUnder;
     wattr.override_redirect = True;
     wattr.save_under = True;
@@ -372,97 +409,77 @@ OXPopupMenu::~OXPopupMenu() {
   if (_selbackGC != _defaultSelBckgndGC) delete _selbackGC;
   if (_font != _defaultFont) _client->FreeFont((OXFont *) _font);
   if (_hifont != _hilightFont) _client->FreeFont((OXFont *) _hifont);
+  if (_sidePic) {
+    _client->FreePicture(_sidePic);
+    _client->FreeColor(_sideBgnd);
+  }
 }
 
 void OXPopupMenu::AddEntry(OHotString *s, int ID, const OPicture *p) {
-  OMenuEntry *nw;
+  OMenuEntry *e;
   int hotchar;
 
-  nw = new OMenuEntry;
+  e = new OMenuEntry(MENU_ENTRY, ID, s, p);
 
-  nw->_label   = s;
-  nw->_pic     = p;
-  nw->_type    = MENU_ENTRY;
-  nw->_entryID = ID;
-  nw->_popup   = NULL;
-  nw->_status  = MENU_ENABLE_MASK;
-  nw->_ex  = 2;
-  nw->_ey  = _h-2;
-  nw->next = NULL;
-  nw->prev = NULL;
+  e->_ex = _sidePic ? _sidePic->GetWidth() + 4 : 2;
+  e->_ey = _h - 2;
 
   if ((hotchar = s->GetHotChar()) != 0)
-    nw->_hkeycode = XKeysymToKeycode(GetDisplay(), hotchar);
+    e->_hkeycode = XKeysymToKeycode(GetDisplay(), hotchar);
   else
-    nw->_hkeycode = 0;
+    e->_hkeycode = 0;
 
   if (_last == NULL) {
-    _first = _last = nw;
+    _first = _last = e;
   } else {
-    _last->next = nw;
-    nw->prev = _last;
-    _last = nw;
+    _last->next = e;
+    e->prev = _last;
+    _last = e;
   }
 
   AdjustSize();
 }
 
-void OXPopupMenu::AddSeparator() {
-  OMenuEntry *nw;
+void OXPopupMenu::AddSeparator(int ID) {
+  OMenuEntry *e;
 
-  nw = new OMenuEntry;
+  e = new OMenuEntry(MENU_SEPARATOR, ID);
 
-  nw->_label   = NULL;
-  nw->_pic     = NULL;
-  nw->_type    = MENU_SEPARATOR;
-  nw->_entryID = -1;
-  nw->_popup   = NULL;
-  nw->_status  = MENU_ENABLE_MASK;
-  nw->_ex  = 2;
-  nw->_ey  = _h-2;
-  nw->_hkeycode = 0;
-  nw->next = NULL;
-  nw->prev = NULL;
+  e->_ex = _sidePic ? _sidePic->GetWidth() + 4 : 2;
+  e->_ey = _h - 2;
 
   if (_last == NULL) {
-    _first = _last = nw;
+    _first = _last = e;
   } else {
-    _last->next = nw;
-    nw->prev = _last;
-    _last = nw;
+    _last->next = e;
+    e->prev = _last;
+    _last = e;
   }
 
   AdjustSize();
 }
 
-void OXPopupMenu::AddPopup(OHotString *s, OXPopupMenu *popup) {
-  OMenuEntry *nw;
+void OXPopupMenu::AddPopup(OHotString *s, OXPopupMenu *popup, int ID) {
+  OMenuEntry *e;
   int hotchar;
 
-  nw = new OMenuEntry;
+  e = new OMenuEntry(MENU_POPUP, ID, s);
 
-  nw->_label   = s;
-  nw->_pic     = NULL;
-  nw->_type    = MENU_POPUP;
-  nw->_entryID = -2;
-  nw->_popup   = popup;
-  nw->_status  = MENU_ENABLE_MASK;
-  nw->_ex  = 2;
-  nw->_ey  = _h-2;
-  nw->next = NULL;
-  nw->prev = NULL;
+  e->_popup = popup;
+  e->_ex = _sidePic ? _sidePic->GetWidth() + 4 : 2;
+  e->_ey = _h - 2;
 
   if ((hotchar = s->GetHotChar()) != 0)
-    nw->_hkeycode = XKeysymToKeycode(GetDisplay(), hotchar);
+    e->_hkeycode = XKeysymToKeycode(GetDisplay(), hotchar);
   else
-    nw->_hkeycode = 0;
+    e->_hkeycode = 0;
 
   if (_last == NULL) {
-    _first = _last = nw;
+    _first = _last = e;
   } else {
-    _last->next = nw;
-    nw->prev = _last;
-    _last = nw;
+    _last->next = e;
+    e->prev = _last;
+    _last = e;
   }
 
   if (popup) popup->Associate(_msgObject);
@@ -470,10 +487,55 @@ void OXPopupMenu::AddPopup(OHotString *s, OXPopupMenu *popup) {
   AdjustSize();
 }
 
-void OXPopupMenu::RemoveEntry(int ID) {
+int OXPopupMenu::RenameEntry(int ID, OHotString *s) {
+  OMenuEntry *ptr;
+  int hotchar;
+
+  if (!s) return False;
+
+  for (ptr = _first; ptr; ptr = ptr->next) {
+    if (ptr->_entryID == ID) break;
+  }
+
+  if (!ptr || (ptr->_type == MENU_SEPARATOR)) return False;
+
+  if (ptr->_label) delete ptr->_label;
+  ptr->_label = s;
+
+  if ((hotchar = s->GetHotChar()) != 0)
+    ptr->_hkeycode = XKeysymToKeycode(GetDisplay(), hotchar);
+  else
+    ptr->_hkeycode = 0;
+
+  AdjustSize();
+
+  return True;
+}
+
+int OXPopupMenu::SetEntryPic(int ID, const OPicture *p) {
+  OMenuEntry *ptr;
+  int hotchar;
+
+  for (ptr = _first; ptr; ptr = ptr->next) {
+    if (ptr->_entryID == ID) break;
+  }
+
+  if (!ptr || (ptr->_type == MENU_SEPARATOR)) return False;
+
+  //if (ptr->_pic) _client->FreePicture(ptr->_pic);
+  ptr->_pic = p;
+
+  AdjustSize();
+
+  return True;
+}
+
+void OXPopupMenu::RemoveEntry(int ID, int type) {
   OMenuEntry *ptr, *ptr2;
 
-  for (ptr = _first; ptr && ptr->_entryID != ID; ptr = ptr->next);
+  for (ptr = _first; ptr; ptr = ptr->next) {
+    if ((ptr->_entryID == ID) && (ptr->_type == type)) break;
+  }
   if (ptr == NULL) return; // not found!
 
   if (ptr->prev == NULL) {
@@ -492,6 +554,7 @@ void OXPopupMenu::RemoveEntry(int ID) {
     ptr2 = ptr->next;
     ptr2->prev = ptr->prev;
   }
+  if (ptr == _current) _current = NULL;
   delete ptr;
 
   AdjustSize();
@@ -507,6 +570,18 @@ void OXPopupMenu::RemoveAllEntries() {
     ptr = next;
   }
   _first = _last = NULL;
+  _current = NULL;
+
+  AdjustSize();
+}
+
+void OXPopupMenu::SetSidePic(const OPicture *p, unsigned long color) {
+  if (_sidePic) {
+    _client->FreePicture(_sidePic);
+    _client->FreeColor(_sideBgnd);
+  }
+  _sidePic = p;
+  _sideBgnd = color;
 
   AdjustSize();
 }
@@ -517,19 +592,27 @@ void OXPopupMenu::AdjustSize() {
 }
 
 void OXPopupMenu::AdjustEntries() {
-  int lsize, rsize, sepw, tw, ltw, rtw, pw, w, h;
+  int lsize, rsize, sepw, tw, ltw, rtw, x0, spw, pw, w, h;
   const char *lstr, *rstr;
   OMenuEntry *ptr;
 
-  w = 8;
+  if (_sidePic) {
+    x0 = _sidePic->GetWidth() + 4;
+    spw = _sidePic->GetWidth();
+    w = spw + 8;
+  } else {
+    x0 = 2;
+    spw = 0;
+    w = 8;
+  }
   h = 6;
-  _xl = _xr = 16;
-  sepw = 10;
+  _xl = _xr = LEFT_GAP;
+  sepw = TEXT_GAP;
   lsize = rsize = 0;
 
-  for (ptr=_first; ptr!=NULL; ptr=ptr->next) {
-    ptr->_ex = 2;
-    ptr->_ey = h-2;
+  for (ptr = _first; ptr != NULL; ptr = ptr->next) {
+    ptr->_ex = x0;
+    ptr->_ey = h - 2;
     switch (ptr->_type) {
       case MENU_ENTRY:
         lstr = ptr->_label->GetString();
@@ -543,15 +626,15 @@ void OXPopupMenu::AdjustEntries() {
           rtw = 0;
         }
         if (ptr->_pic) {
-          pw = ptr->_pic->GetWidth();
-          if (pw+12 > _xl) { w += pw+12-_xl; _xl = pw+12; }
+          pw = ptr->_pic->GetWidth() + PIC_GAP;
+          if (pw + LEFT_GAP > _xl) _xl = pw + LEFT_GAP;
         } else {
           pw = 0;
         }
         lsize = max(lsize, ltw);
         rsize = max(rsize, rtw);
         tw = (rsize > 0) ? lsize + rsize + sepw : ltw;
-        w = max(w, tw + pw /*+8*/+18+12);
+        w = max(w, spw + _xl + tw + RIGHT_GAP); // _xl already includes pw
         h += _hifont->TextHeight() + 3;
         break;
 
@@ -567,7 +650,7 @@ void OXPopupMenu::AdjustEntries() {
           rtw = 0;
         }
         tw = (rsize > 0) ? lsize + rsize + sepw : ltw;
-        w = max(w, tw +8+18+12);
+        w = max(w, spw + _xl + tw + RIGHT_GAP + POPUPMARK_WIDTH);
         h += _hifont->TextHeight() + 3;
         break;
 
@@ -595,26 +678,27 @@ void OXPopupMenu::PlaceMenu(int x, int y, int stick_mode, int grab_pointer) {
   Window wdummy;
   
   _stick = stick_mode;
-  Activate(NULL); // _current = NULL;
+  Activate(NULL);
+  _current = NULL;
 
   // Parent is root window for a popup menu:
   XGetGeometry(GetDisplay(), _parent->GetId(), &wdummy,
                &rx, &ry, &rw, &rh, &dummy, &dummy);
 
+  if (x + _w > rw) x -= _w;  //x = rw - _w;
   if (x < 0) x = 0;
-  if (x + _w > rw) x = rw - _w;
+  if (y + _h > rh) y -= _h;  //y = rh - _h;
   if (y < 0) y = 0;
-  if (y + _h > rh) y = rh - _h;
 
   Move(x, y);
   MapRaised();
 
   if (grab_pointer) {
-    XGrabPointer(GetDisplay(), _id, False/*<<<Onyx True*/,
+    XGrabPointer(GetDisplay(), _id, False,
  		 ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
 		 GrabModeAsync, GrabModeAsync, 
                  None, GetResourcePool()->GetGrabCursor(), CurrentTime);
-    XGrabKeyboard(GetDisplay(), _id, False/*True*/,
+    XGrabKeyboard(GetDisplay(), _id, False,
 		  GrabModeAsync, GrabModeAsync,
                   CurrentTime);
     _hasgrab = True;
@@ -626,13 +710,10 @@ void OXPopupMenu::PlaceMenu(int x, int y, int stick_mode, int grab_pointer) {
 int OXPopupMenu::PopupMenu(int x, int y) {
   PlaceMenu(x, y, True, True);
   _client->WaitForUnmap(this);
-  if (_current && _current->_status & MENU_ENABLE_MASK)
-    return _current->_entryID;
-  return -1;
+  return _lastID;
 }
 
 int OXPopupMenu::EndMenu() {
-  int ID;
 
   if (_delay) { delete _delay; _delay = NULL; }
 
@@ -644,28 +725,29 @@ int OXPopupMenu::EndMenu() {
     _current->_status &= ~MENU_ACTIVE_MASK;
 
     if (_current->_type == MENU_POPUP) {
-      ID = _current->_popup->EndMenu();
-    } else {
+      _lastID = _current->_popup->EndMenu();
+    } else if (_current->_type == MENU_ENTRY) {
       // return the ID if the entry is enabled, otherwise -1
       if (_current->_status & MENU_ENABLE_MASK) {
-        ID = _current->_entryID;
+        _lastID = _current->_entryID;
       } else {
-        ID = -1;
+        _lastID = -1;
       }
+    } else {  // separator
+      _lastID = -1;
     }
 
   } else {
     // if no entry selected...
-    ID = -1;
+    _lastID = -1;
   }
 
   // then unmap itself
   UnmapWindow();
-  return ID;
+  return _lastID;
 }
 
 int OXPopupMenu::HandleButton(XButtonEvent *event) {
-  int ID;
   
   if (event->type == ButtonRelease) {
     if (_stick) {
@@ -673,28 +755,11 @@ int OXPopupMenu::HandleButton(XButtonEvent *event) {
       return True;
     }
 
-//// OnyX
-/*
-  if (_current && _current->_type == MENU_POPUP && _popdown) {
-    OXPopupMenu *p = _current->_popup;
-    if (p) {
-      int retc = p->HandleButton(event);
-      if (retc) return retc;
-    }
-  }
-  if (!(event->x_root >= _x &&
-        event->x_root <  _x+_w &&
-        event->y_root >= _y &&
-        event->y_root <  _y+_h)) return False;
-*/
-//// OnyX
-
-//    if (_current && _current->_type == MENU_POPUP) return True;
-    ID = EndMenu();
+    int ID = EndMenu();
     if (_hasgrab) {
       XUngrabPointer(GetDisplay(), CurrentTime);
       XUngrabKeyboard(GetDisplay(), CurrentTime);
-      XFlush(GetDisplay());
+      XSync(GetDisplay(), False);
     }
     if (_current) {
       _current->_status &= ~MENU_ACTIVE_MASK;
@@ -707,29 +772,6 @@ int OXPopupMenu::HandleButton(XButtonEvent *event) {
   return True;
 }
 
-int OXPopupMenu::HandleCrossing(XCrossingEvent *event) {
-
-  if (event->type == EnterNotify) {
-    OMenuEntry *ptr;
-    int h;
-
-    for (ptr = _first; ptr != NULL; ptr=ptr->next) {
-      if (ptr->_type == MENU_SEPARATOR) {
-        h = 4;
-      } else {
-        h = _hifont->TextHeight() + 3 +2;
-      }
-      if ((event->x >= ptr->_ex) && (event->x <= ptr->_ex+_w-10) &&
-          (event->y >= ptr->_ey) && (event->y <= ptr->_ey+h)) break;
-    }
-    Activate(ptr);
-  } else {
-    Activate(NULL);
-  }
-
-  return True;
-}
-
 int OXPopupMenu::HandleMotion(XMotionEvent *event) {
   OMenuEntry *ptr;
   int h;
@@ -737,7 +779,6 @@ int OXPopupMenu::HandleMotion(XMotionEvent *event) {
   OXFrame::HandleMotion(event);
   _stick = False;
 
-//// OnyX
   if (_current && _current->_type == MENU_POPUP && _popdown) {
     OXPopupMenu *p = _current->_popup;
     if (p) {
@@ -746,20 +787,16 @@ int OXPopupMenu::HandleMotion(XMotionEvent *event) {
       ev.y = ev.y_root - p->GetY();
       int retc = p->HandleMotion(&ev);
       if (retc) return retc;
+      if (!Contains(event->x, event->y)) return False;
     }
   }
 
-#if 0
-  if (!(event->x_root >= _x &&
-        event->x_root <  _x+_w &&
-        event->y_root >= _y &&
-        event->y_root <  _y+_h)) return False;
-#else
-  if (!Contains(event->x, event->y)) { Activate(NULL); return False; }
-#endif
-//// OnyX
+  if (!Contains(event->x, event->y)) {
+    Activate(NULL);
+    return False;
+  }
 
-  for (ptr = _first; ptr != NULL; ptr=ptr->next) {
+  for (ptr = _first; ptr != NULL; ptr = ptr->next) {
     if (ptr->_type == MENU_SEPARATOR) {
       h = 4;
     } else {
@@ -817,7 +854,7 @@ int OXPopupMenu::HandleKey(XKeyEvent *event) {
       int ID = EndMenu();
       XUngrabPointer(GetDisplay(), CurrentTime);
       XUngrabKeyboard(GetDisplay(), CurrentTime);
-      XFlush(GetDisplay());
+      XSync(GetDisplay(), False);
       if (ptr) {
         ptr->_status &= ~MENU_ACTIVE_MASK;
         if (ptr->_status & MENU_ENABLE_MASK) {
@@ -859,11 +896,13 @@ int OXPopupMenu::HandleKey(XKeyEvent *event) {
 
       case XK_Right:
         if (!_popdown && _current && _current->_type == MENU_POPUP) {
-          Activate(_current, False);
-          _popdown = True;
-          OXPopupMenu *p = _current->_popup;
-          if (p) p->Activate(p->_first);
-          return True;
+          if (_current->_status & MENU_ENABLE_MASK) {
+            Activate(_current, False);
+            _popdown = True;
+            OXPopupMenu *p = _current->_popup;
+            if (p) p->Activate(p->_first);
+            return True;
+          }
         }
         return False;
         break;
@@ -916,13 +955,12 @@ void OXPopupMenu::Activate(OMenuEntry *entry, int delayed) {
 
 //  if (entry == _current) return;
 
-  _popdown = False;
-
   //-- Deactivate the current entry
 
   if (_current && (_current != entry)) {
     if (entry == NULL && _current->_type == MENU_POPUP) return;
     if (_current->_type == MENU_POPUP) _current->_popup->EndMenu();
+    _popdown = False;
     _current->_status &= ~MENU_ACTIVE_MASK;
     DrawEntry(_current);
   }
@@ -934,7 +972,7 @@ void OXPopupMenu::Activate(OMenuEntry *entry, int delayed) {
   if (entry) {
     entry->_status |= MENU_ACTIVE_MASK;
     DrawEntry(entry);
-    if (entry->_type == MENU_POPUP) {
+    if ((entry->_type == MENU_POPUP) && (entry->_status & MENU_ENABLE_MASK)) {
       if (delayed) {
         _delay = new OTimer(this, 350);
       } else {
@@ -943,10 +981,17 @@ void OXPopupMenu::Activate(OMenuEntry *entry, int delayed) {
 
         XTranslateCoordinates(GetDisplay(), _id,
                               (_current->_popup->GetParent())->GetId(),
-                              _current->_ex+_w, _current->_ey,
-                              &ax, &ay, &wdummy);
+                              _w, _current->_ey, &ax, &ay, &wdummy);
 
-        _current->_popup->PlaceMenu(ax-5, ay-1, False, False);
+        ODimension ps = _current->_popup->GetDefaultSize();
+
+        if (ax + ps.w > _client->GetDisplayWidth())
+          ax -= _w + ps.w - 7;
+
+        if (ay + ps.h > _client->GetDisplayHeight())
+          ay = _client->GetDisplayHeight() - ps.h - 3;
+
+        _current->_popup->PlaceMenu(ax-3, ay-1, False, False);
         _popdown = True;
       }
     } else if (entry->_type == MENU_ENTRY) {
@@ -962,19 +1007,27 @@ void OXPopupMenu::Activate(OMenuEntry *entry, int delayed) {
 int OXPopupMenu::HandleTimer(OTimer *t) {
   if (t != _delay) return False;
 
-  if (_current != NULL)
-    if (_current->_type == MENU_POPUP) {
-      int ax, ay;
-      Window wdummy;
+  if (_current && 
+      (_current->_type == MENU_POPUP) &&
+      (_current->_status & MENU_ENABLE_MASK)) {
+    int ax, ay;
+    Window wdummy;
 
-      XTranslateCoordinates(GetDisplay(), _id,
-                            (_current->_popup->GetParent())->GetId(),
-                            _current->_ex+_w, _current->_ey,
-                            &ax, &ay, &wdummy);
+    XTranslateCoordinates(GetDisplay(), _id,
+                          (_current->_popup->GetParent())->GetId(),
+                          _w, _current->_ey, &ax, &ay, &wdummy);
 
-      _current->_popup->PlaceMenu(ax-5, ay-1, False, False);
-      _popdown = True;
-    }
+    ODimension ps = _current->_popup->GetDefaultSize();
+
+    if (ax + ps.w > _client->GetDisplayWidth())
+      ax -= _w + ps.w - 7;
+
+    if (ay + ps.h > _client->GetDisplayHeight())
+      ay = _client->GetDisplayHeight() - ps.h - 3;
+
+    _current->_popup->PlaceMenu(ax-3, ay-1, False, False);
+    _popdown = True;
+  }
 
   delete _delay;
   _delay = NULL;
@@ -986,6 +1039,26 @@ void OXPopupMenu::_DoRedraw() {
   OMenuEntry *ptr;
   
   OXFrame::_DoRedraw();
+
+  if (_sidePic) {
+    unsigned long fg = _normGC->GetForeground();
+    _normGC->SetForeground(_sideBgnd);
+    FillRectangle(_normGC->GetGC(), 3, 3, _sidePic->GetWidth(), _h - 6);
+
+    _normGC->SetClipMask(_sidePic->GetMask());
+    int clip_x = 3;
+    int clip_y = _h - _sidePic->GetHeight() - 3;
+    _normGC->SetClipXOrigin(clip_x);
+    _normGC->SetClipYOrigin(clip_y);
+
+    XCopyArea(GetDisplay(), _sidePic->GetPicture(), _id, _normGC->GetGC(),
+              0, 0, _sidePic->GetWidth(), _sidePic->GetHeight(),
+              clip_x, clip_y);
+
+    _normGC->SetClipMask(None);
+    _normGC->SetForeground(fg);
+  }
+
   for (ptr = _first; ptr != NULL; ptr = ptr->next) {
     DrawEntry(ptr);
   }
@@ -1010,6 +1083,8 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
   int tr = entry->_ex + _xr;
   int ty = entry->_ey + fm.ascent;
   int th = font->TextHeight() + 3;
+  int px = entry->_ex + LEFT_GAP;
+  int ew = _w - 6 - (_sidePic ? _sidePic->GetWidth() + 2 : 0);
 
   GC sGC  = _selGC->GetGC();
   GC sbGC = _selbackGC->GetGC();
@@ -1023,20 +1098,16 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
 
       if (entry->_status & MENU_ACTIVE_MASK) {
         XFillRectangle(GetDisplay(), _id, sbGC,
-                       entry->_ex+1, entry->_ey-1, _w-6, th);
+                       entry->_ex+1, entry->_ey-1, ew, th);
         if (entry->_type == MENU_POPUP)
           DrawTrianglePattern(sGC, _w-10, entry->_ey+3, _w-6, entry->_ey+11);
         if (entry->_status & MENU_CHECKED_MASK)
           DrawCheckMark(sGC, 6, entry->_ey+3, 14, entry->_ey+11);
         if (entry->_status & MENU_RCHECKED_MASK)
           DrawRCheckMark(sGC, 6, entry->_ey+3, 14, entry->_ey+11);
-        if (entry->_pic != NULL)
-          entry->_pic->Draw(GetDisplay(), _id, sGC, 8, entry->_ey+1);
-#if 0
-        entry->_label->Draw(GetDisplay(), _id,
-                       (entry->_status & MENU_ENABLE_MASK) ? sGC : _shadowGC,
-                       tx, ty);
-#else
+        if (entry->_pic)
+          entry->_pic->Draw(GetDisplay(), _id, sGC, px, entry->_ey+1);
+
         if (rstr) {
           entry->_label->Draw(GetDisplay(), _id,
                        (entry->_status & MENU_ENABLE_MASK) ? sGC : _shadowGC,
@@ -1049,10 +1120,10 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
                        (entry->_status & MENU_ENABLE_MASK) ? sGC : _shadowGC,
                        tx, ty);
         }
-#endif
+
       } else {
-//        XFillRectangle(GetDisplay(), _id, _bckgndGC,
-        XClearArea(GetDisplay(), _id, entry->_ex+1, entry->_ey-1, _w-6, th,false);
+//        FillRectangle(_bckgndGC,
+        ClearArea(entry->_ex+1, entry->_ey-1, ew, th, False);
 
         if (entry->_type == MENU_POPUP)
           DrawTrianglePattern(nGC, _w-10, entry->_ey+3, _w-6, entry->_ey+11);
@@ -1060,16 +1131,9 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
           DrawCheckMark(nGC, 6, entry->_ey+3, 14, entry->_ey+11);
         if (entry->_status & MENU_RCHECKED_MASK)
           DrawRCheckMark(nGC, 6, entry->_ey+3, 14, entry->_ey+11);
-        if (entry->_pic != NULL)
-          entry->_pic->Draw(GetDisplay(), _id, nGC, 8, entry->_ey+1);
-#if 0
-        if (entry->_status & MENU_ENABLE_MASK) {
-          entry->_label->Draw(GetDisplay(), _id, nGC, tx, ty);
-        } else {
-          entry->_label->Draw(GetDisplay(), _id, _hilightGC, tx+1, ty+1);
-          entry->_label->Draw(GetDisplay(), _id, _shadowGC, tx, ty);
-        }
-#else
+        if (entry->_pic)
+          entry->_pic->Draw(GetDisplay(), _id, nGC, px, entry->_ey+1);
+
         if (rstr) {
           if (entry->_status & MENU_ENABLE_MASK) {
             entry->_label->Draw(GetDisplay(), _id, nGC, tx, ty, rstr - lstr);
@@ -1090,13 +1154,13 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
             entry->_label->Draw(GetDisplay(), _id, _shadowGC, tx, ty);
           }
         }
-#endif
+
       }
       break;
 
    case MENU_SEPARATOR:
-      XDrawLine(GetDisplay(), _id, _shadowGC,  2, entry->_ey, _w-3, entry->_ey);
-      XDrawLine(GetDisplay(), _id, _hilightGC, 2, entry->_ey+1, _w-3, entry->_ey+1);
+      DrawLine(_shadowGC,  entry->_ex, entry->_ey, _w-3, entry->_ey);
+      DrawLine(_hilightGC, entry->_ex, entry->_ey+1, _w-3, entry->_ey+1);
       break;
    }
 
@@ -1108,15 +1172,15 @@ void OXPopupMenu::DrawEntry(OMenuEntry *entry) {
 }
 
 void OXPopupMenu::DrawBorder() {
-  XDrawLine (GetDisplay(), _id, _bckgndGC, 0, 0, _w-2, 0);
-  XDrawLine (GetDisplay(), _id, _bckgndGC, 0, 0, 0, _h-2);
-  XDrawLine (GetDisplay(), _id, _hilightGC, 1, 1, _w-3, 1);
-  XDrawLine (GetDisplay(), _id, _hilightGC, 1, 1, 1, _h-3);
+  DrawLine(_bckgndGC, 0, 0, _w-2, 0);
+  DrawLine(_bckgndGC, 0, 0, 0, _h-2);
+  DrawLine(_hilightGC, 1, 1, _w-3, 1);
+  DrawLine(_hilightGC, 1, 1, 1, _h-3);
     
-  XDrawLine (GetDisplay(), _id, _shadowGC,  1, _h-2, _w-2, _h-2);
-  XDrawLine (GetDisplay(), _id, _shadowGC,  _w-2, _h-2, _w-2, 1);
-  XDrawLine (GetDisplay(), _id, _blackGC, 0, _h-1, _w-1, _h-1);
-  XDrawLine (GetDisplay(), _id, _blackGC, _w-1, _h-1, _w-1, 0);
+  DrawLine(_shadowGC,  1, _h-2, _w-2, _h-2);
+  DrawLine(_shadowGC,  _w-2, _h-2, _w-2, 1);
+  DrawLine(_blackGC, 0, _h-1, _w-1, _h-1);
+  DrawLine(_blackGC, _w-1, _h-1, _w-1, 0);
 }
 
 void OXPopupMenu::DrawTrianglePattern(GC gc, int l, int t, int r, int b) {
@@ -1253,6 +1317,28 @@ int OXPopupMenu::IsEntryRChecked(int ID) {
   return 0;
 }
 
+void OXPopupMenu::Reconfig() {
+  OMenuEntry *ptr;
+
+  _backPixel = _defaultFrameBackground;
+  OXFrame::Reconfig();
+  SetBackgroundColor(_backPixel);
+
+  _normGC->SetForeground(_client->GetResourcePool()->GetFrameFgndColor());
+  _normGC->SetBackground(_client->GetResourcePool()->GetFrameBgndColor());
+
+  _selGC->SetForeground(_client->GetResourcePool()->GetSelectedFgndColor());
+  _selGC->SetBackground(_client->GetResourcePool()->GetSelectedBgndColor());
+
+  _selbackGC->SetForeground(_client->GetResourcePool()->GetSelectedBgndColor());
+  _selbackGC->SetBackground(_client->GetResourcePool()->GetSelectedFgndColor());
+
+  for (ptr = _first; ptr != NULL; ptr = ptr->next)
+    if (ptr->_type == MENU_POPUP) ptr->_popup->Reconfig();
+
+  AdjustEntries();
+  Resize(_mw, _mh);
+}
 
 //-----------------------------------------------------------------
 
@@ -1301,18 +1387,27 @@ OXMenuTitle::~OXMenuTitle() {
 void OXMenuTitle::SetState(int state) {
   _state = state;
   if (state) {
-    if (_menu != NULL) {
-      int ax, ay;
+    if (_menu) {
+      int ax, ay, dpyw, dpyh;
       Window wdummy;
 
       XTranslateCoordinates(GetDisplay(), _id, (_menu->GetParent())->GetId(),
-			0, 0, &ax, &ay, &wdummy);
+                            0, _h, &ax, &ay, &wdummy);
 
-      // Place the menu just under the window :
-      _menu->PlaceMenu(ax-1, ay+_h, True, False); //True);
+      ODimension ps = _menu->GetDefaultSize();
+
+      dpyw = _client->GetDisplayWidth();
+      dpyh = _client->GetDisplayHeight();
+
+      if (ax + ps.w > dpyw) ax -= ps.w - _w - 2;
+      if (ax + ps.w > dpyw) ax = dpyw - ps.w - 1;  // if still off-screen
+      if (ay + ps.h > dpyh) ay -= ps.h + _h;
+
+      // Place the menu just under the window:
+      _menu->PlaceMenu(ax-1, ay, True, False);
     }
   } else {
-    if (_menu != NULL) {
+    if (_menu) {
       _ID = _menu->EndMenu();
     }
   }
@@ -1338,7 +1433,7 @@ void OXMenuTitle::_DoRedraw() {
 }
 
 void OXMenuTitle::DoSendMessage() {
-  if (_menu != NULL)
+  if (_menu)
     if (_ID != -1) {
       OMenuMessage message(MSG_MENU, MSG_CLICK, _ID);
       SendMessage(_menu->_msgObject, &message);
@@ -1350,8 +1445,6 @@ int OXMenuTitle::HandleKey(XKeyEvent *event) {
   return False;
 }
 
-//// OnyX:
-
 int OXMenuTitle::HandleButton(XButtonEvent *event) {
   if (_menu && _state) return _menu->HandleButton(event);
   return False;
@@ -1367,3 +1460,8 @@ int OXMenuTitle::HandleMotion(XMotionEvent *event) {
   }
   return False;
 }
+
+void OXMenuTitle::Reconfig() {
+  OXFrame::Reconfig();
+  if (_menu) _menu->Reconfig();
+}
