@@ -45,6 +45,10 @@
 #include <xclass/utils.h>
 #include <xclass/version.h>
 
+#include "menu_file.xpm"
+#include "menu_test.xpm"
+#include "menu_help.xpm"
+
 
 #define M_FILE_OPEN      1
 #define M_FILE_SAVE      2
@@ -82,7 +86,7 @@ public:
   OXTestMainFrame(const OXWindow *p, int w, int h);
   virtual ~OXTestMainFrame();
 
-virtual int ProcessMessage(OMessage *msg);
+  virtual int ProcessMessage(OMessage *msg);
 
 protected:
   OXCompositeFrame *_statusFrame;
@@ -104,8 +108,8 @@ public:
                unsigned long options = MAIN_FRAME | VERTICAL_FRAME);
   virtual ~OXTestDialog();
 
-  virtual void CloseWindow();
   virtual int ProcessMessage(OMessage *msg);
+
 protected:
   OXCompositeFrame *Frame1, *f1, *f2, *f3, *f4;
   OXButton *OkButton, *CancelButton, *btn1, *btn2, *chk1, *chk2, *rad1, *rad2;
@@ -121,7 +125,6 @@ public:
                unsigned long options = MAIN_FRAME | VERTICAL_FRAME);
   virtual ~OXTestMsgBox();
 
-  virtual void CloseWindow();
   virtual int ProcessMessage(OMessage *msg);
 
 protected:
@@ -192,6 +195,8 @@ OXTestMainFrame::OXTestMainFrame(const OXWindow *p, int w, int h) :
   _menuBarHelpLayout = new OLayoutHints(LHINTS_TOP | LHINTS_RIGHT);
 
   _menuFile = new OXPopupMenu(_client->GetRoot());
+  _menuFile->SetSidePic(_client->GetPicture("menu_file.xpm", menu_file_xpm),
+                        _client->GetColorByName("#808080"));
   _menuFile->AddEntry(new OHotString("&Open..."), M_FILE_OPEN);
   _menuFile->AddEntry(new OHotString("&Save"), M_FILE_SAVE);
   _menuFile->AddEntry(new OHotString("S&ave as..."), M_FILE_SAVEAS);
@@ -225,6 +230,8 @@ OXTestMainFrame::OXTestMainFrame(const OXWindow *p, int w, int h) :
   _cascadeMenu->AddPopup(new OHotString("C&ascaded 2"), _cascade2Menu);
 
   _menuTest = new OXPopupMenu(_client->GetRoot());
+  _menuTest->SetSidePic(_client->GetPicture("menu_test.xpm", menu_test_xpm),
+                        _client->GetColorByName("#808080"));
   _menuTest->AddEntry(new OHotString("&Dialog..."), M_TEST_DLG);
   _menuTest->AddEntry(new OHotString("&Message Box..."), M_TEST_MSGBOX);
   _menuTest->AddEntry(new OHotString("&Sliders..."), M_TEST_SLIDER);
@@ -232,6 +239,8 @@ OXTestMainFrame::OXTestMainFrame(const OXWindow *p, int w, int h) :
   _menuTest->AddPopup(new OHotString("&Cascaded menus"), _cascadeMenu);
 
   _menuHelp = new OXPopupMenu(_client->GetRoot());
+  _menuHelp->SetSidePic(_client->GetPicture("menu_help.xpm", menu_help_xpm),
+                        _client->GetColorByName("#808080"));
   _menuHelp->AddEntry(new OHotString("&Contents"), M_HELP_CONTENTS);
   _menuHelp->AddEntry(new OHotString("&Search..."), M_HELP_SEARCH);
   _menuHelp->AddSeparator();
@@ -320,7 +329,7 @@ int OXTestMainFrame::ProcessMessage(OMessage *msg) {
 
     case MSG_BUTTON:
     case MSG_MENU:
-      switch(msg->action) {
+      switch (msg->action) {
 
         case MSG_SELECT:
           Debug(DBG_INFO, "Pointer over menu entry, ID=%d\n", wmsg->id);
@@ -333,7 +342,7 @@ int OXTestMainFrame::ProcessMessage(OMessage *msg) {
               new OXTestMsgBox(clientX->GetRoot(), this, 400, 200);
             break;
           }
-          switch(wmsg->id) {
+          switch (wmsg->id) {
 
             case M_FILE_OPEN:
               {
@@ -362,9 +371,7 @@ int OXTestMainFrame::ProcessMessage(OMessage *msg) {
 
             case M_FILE_EXIT:
               Debug(DBG_INFO, "\nM_FILE_EXIT\n");
-              delete this; // mainWindow;
-              delete clientX;
-              exit(0);
+              CloseWindow();
               break;
 
             case M_TEST_MSGBOX:
@@ -413,19 +420,19 @@ int OXTestMainFrame::ProcessMessage(OMessage *msg) {
             default:
               Debug(DBG_INFO, "\nID: %d\n", wmsg->id);
               break;
-          } // switch(msg->id)
+          } // switch (msg->id)
           break;
 
         default:
           break;
 
-      } // switch(msg->action)
+      } // switch (msg->action)
       break;
 
     default:
       break;
 
-  } // switch(msg->type)
+  } // switch (msg->type)
 
   return True;
 }
@@ -533,19 +540,15 @@ OXTestDialog::~OXTestDialog() {
   delete L3;
 }
 
-void OXTestDialog::CloseWindow() {
-  delete this;
-}
-
 int OXTestDialog::ProcessMessage(OMessage *msg) {
   OWidgetMessage *wmsg = (OWidgetMessage *) msg;
 
-  switch(msg->type) {
+  switch (msg->type) {
 
     case MSG_BUTTON:
-      switch(msg->action) {
+      switch (msg->action) {
         case MSG_CLICK:
-          switch(wmsg->id) {
+          switch (wmsg->id) {
             case 1:
               _tab->RemoveTab(1);
               break;
@@ -562,9 +565,9 @@ int OXTestDialog::ProcessMessage(OMessage *msg) {
       break;
 
     case MSG_RADIOBUTTON:
-      switch(msg->action) {
+      switch (msg->action) {
         case MSG_CLICK:
-          switch(wmsg->id) {
+          switch (wmsg->id) {
             case 81:
               rad2->SetState(BUTTON_UP);
               break;
@@ -722,10 +725,6 @@ OXTestMsgBox::~OXTestMsgBox() {
   delete L21;
 }
 
-void OXTestMsgBox::CloseWindow() {
-  delete this;
-}
-
 int OXTestMsgBox::ProcessMessage(OMessage *msg) {
   OWidgetMessage *wmsg = (OWidgetMessage *) msg;
   int i, buttons, icontype, retval;
@@ -733,12 +732,12 @@ int OXTestMsgBox::ProcessMessage(OMessage *msg) {
   OString stitle(tbtitle->GetString());
   OString smsg(tbmsg->GetString());
 
-  switch(msg->action) {
+  switch (msg->action) {
     case MSG_CLICK:
 
-      switch(msg->type) {
+      switch (msg->type) {
         case MSG_BUTTON:
-          switch(wmsg->id) {
+          switch (wmsg->id) {
             case 1:
               buttons = 0;
               for (i=0; i<10; ++i)
@@ -881,17 +880,16 @@ OXTestSliders::~OXTestSliders() {
 }
 
 int OXTestSliders::ProcessMessage(OMessage *msg) {
-  int rtc;
   char buf[10];
   OTextEntryMessage *temsg;
   OSliderMessage *slmsg;
 
-  switch(msg->type) {
+  switch (msg->type) {
     case MSG_TEXTENTRY:
       temsg = (OTextEntryMessage *) msg;
-      switch(temsg->action) {
+      switch (temsg->action) {
         case MSG_TEXTCHANGED:
-	  switch(temsg->id) {
+	  switch (temsg->id) {
 	    case HId1:
 		hslider1->SetPosition(atoi(tbh1->GetString()));
 		break;
@@ -912,7 +910,7 @@ int OXTestSliders::ProcessMessage(OMessage *msg) {
     case MSG_VSLIDER:
     case MSG_HSLIDER:
       slmsg = (OSliderMessage *) msg;
-      switch(msg->action) {
+      switch (msg->action) {
         case MSG_SLIDERPOS:
 	  sprintf(buf, "%d", slmsg->pos);
 	  switch (slmsg->id) {
@@ -943,5 +941,5 @@ int OXTestSliders::ProcessMessage(OMessage *msg) {
     default:
       break;
   }
-  return rtc;
+  return True;
 }
